@@ -4,20 +4,15 @@ Common implementation of ID, Population, PopulationView and Assembly classes.
 
 These base classes should be sub-classed by the backend-specific classes.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 """
 
-import numpy
+import numpy as np
 import logging
 import operator
 from itertools import chain
-try:
-    basestring
-    reduce
-except NameError:
-    basestring = str
-    from functools import reduce
+from functools import reduce
 from collections import defaultdict
 from pyNN import random, recording, errors, standardmodels, core, space, descriptions
 from pyNN.models import BaseCellType
@@ -85,7 +80,8 @@ class IDMixin(object):
         if self.local:
             self.as_view().set(**parameters)
         else:
-            raise errors.NotLocalError("Cannot set parameters for a cell that does not exist on this node.")
+            raise errors.NotLocalError(
+                "Cannot set parameters for a cell that does not exist on this node.")
 
     def get_parameters(self):
         """Return a dict of all cell parameters."""
@@ -93,7 +89,8 @@ class IDMixin(object):
             parameter_names = self.celltype.get_parameter_names()
             return dict((k, v) for k, v in zip(parameter_names, self.as_view().get(parameter_names)))
         else:
-            raise errors.NotLocalError("Cannot obtain parameters for a cell that does not exist on this node.")
+            raise errors.NotLocalError(
+                "Cannot obtain parameters for a cell that does not exist on this node.")
 
     @property
     def celltype(self):
@@ -109,7 +106,7 @@ class IDMixin(object):
 
         Cell positions are stored in an array in the parent Population.
         """
-        assert isinstance(pos, (tuple, numpy.ndarray))
+        assert isinstance(pos, (tuple, np.ndarray))
         assert len(pos) == 3
         self.parent._set_cell_position(self, pos)
 
@@ -161,14 +158,15 @@ class BasePopulation(object):
             p[2] is equivalent to p.__getitem__(2).
             p[3:6] is equivalent to p.__getitem__(slice(3, 6))
         """
-        if isinstance(index, (int, numpy.integer)):
+        if isinstance(index, (int, np.integer)):
             return self.all_cells[index]
-        elif isinstance(index, (slice, list, numpy.ndarray)):
+        elif isinstance(index, (slice, list, np.ndarray)):
             return self._get_view(index)
         elif isinstance(index, tuple):
             return self._get_view(list(index))
         else:
-            raise TypeError("indices must be integers, slices, lists, arrays or tuples, not %s" % type(index).__name__)
+            raise TypeError(
+                "indices must be integers, slices, lists, arrays or tuples, not %s" % type(index).__name__)
 
     def __len__(self):
         """Return the total number of cells in the population (all nodes)."""
@@ -235,7 +233,8 @@ class BasePopulation(object):
             index = self.id_to_local_index(id)
             return self.initial_values[variable][index]
         else:
-            logger.warning("Variable '{}' is not in initial values, returning 0.0".format(variable))
+            logger.warning(
+                "Variable '{}' is not in initial values, returning 0.0".format(variable))
             return 0.0
 
     def _set_cell_initial_value(self, id, variable, value):
@@ -248,7 +247,7 @@ class BasePopulation(object):
         # doesn't always work correctly if a position is equidistant between
         # two neurons, i.e. 0.5 should be rounded up, but it isn't always.
         # also doesn't take account of periodic boundary conditions
-        pos = numpy.array([position] * self.positions.shape[1]).transpose()
+        pos = np.array([position] * self.positions.shape[1]).transpose()
         dist_arr = (self.positions - pos)**2
         distances = dist_arr.sum(axis=0)
         nearest = distances.argmin()
@@ -262,7 +261,7 @@ class BasePopulation(object):
         assert isinstance(n, int)
         if not rng:
             rng = random.NumpyRNG()
-        indices = rng.permutation(numpy.arange(len(self), dtype=numpy.int))[0:n]
+        indices = rng.permutation(np.arange(len(self), dtype=np.int))[0:n]
         logger.debug("The %d cells selected have indices %s" % (n, indices))
         logger.debug("%s.sample(%s)", self.label, n)
         return self._get_view(indices)
@@ -277,7 +276,7 @@ class BasePopulation(object):
         """
         # if all the cells have the same value for a parameter, should
         # we return just the number, rather than an array?
-        if isinstance(parameter_names, basestring):
+        if isinstance(parameter_names, str):
             parameter_names = (parameter_names,)
             return_list = False
         else:
@@ -291,24 +290,25 @@ class BasePopulation(object):
             parameter_space = self.celltype.reverse_translate(native_parameter_space)
         else:
             parameter_space = self._get_parameters(*parameter_names)
-        parameter_space.evaluate(simplify=simplify)  # what if parameter space is homogeneous on some nodes but not on others?
-                                                     # this also causes problems if the population size matches the number of MPI nodes
+        # what if parameter space is homogeneous on some nodes but not on others?
+        parameter_space.evaluate(simplify=simplify)
+        # this also causes problems if the population size matches the number of MPI nodes
         parameters = dict(parameter_space.items())
         if gather == True and self._simulator.state.num_processes > 1:
             # seems inefficient to do it in a loop - should do as single operation
             for name in parameter_names:
                 values = parameters[name]
-                if isinstance(values, numpy.ndarray):
+                if isinstance(values, np.ndarray):
                     all_values = {self._simulator.state.mpi_rank: values.tolist()}
-                    local_indices = numpy.arange(self.size,)[self._mask_local].tolist()
+                    local_indices = np.arange(self.size,)[self._mask_local].tolist()
                     all_indices = {self._simulator.state.mpi_rank: local_indices}
                     all_values = recording.gather_dict(all_values)
                     all_indices = recording.gather_dict(all_indices)
                     if self._simulator.state.mpi_rank == 0:
                         values = reduce(operator.add, all_values.values())
                         indices = reduce(operator.add, all_indices.values())
-                        idx = numpy.argsort(indices)
-                        values = numpy.array(values)[idx]
+                        idx = np.argsort(indices)
+                        values = np.array(values)[idx]
                 parameters[name] = values
         try:
             values = [parameters[name] for name in parameter_names]
@@ -346,10 +346,13 @@ class BasePopulation(object):
         # TODO: add example using of function of (x,y,z) and Population.position_generator
         if self.local_size > 0:
             if (isinstance(self.celltype, standardmodels.StandardCellType)
-                and any(name in self.celltype.computed_parameters() for name in parameters)):
+                    and any(name in self.celltype.computed_parameters() for name in parameters)
+                    and not isinstance(self.celltype, standardmodels.cells.SpikeSourceArray)):
+                      # the last condition above is a bit of hack to avoid calling expand() unecessarily
                 # need to get existing parameter space of models so we can perform calculations
                 native_names = self.celltype.get_native_names()
-                parameter_space = self.celltype.reverse_translate(self._get_parameters(*native_names))
+                parameter_space = self.celltype.reverse_translate(
+                    self._get_parameters(*native_names))
                 if self.local_size != self.size:
                     parameter_space.expand((self.size,), self._mask_local)
                 parameter_space.update(**parameters)
@@ -360,7 +363,8 @@ class BasePopulation(object):
                                                  self.celltype.__class__)
             if isinstance(self.celltype, standardmodels.StandardCellType):
                 parameter_space = self.celltype.translate(parameter_space)
-            assert parameter_space.shape == (self.size,), "{} != {}".format(parameter_space.shape, self.size)
+            assert parameter_space.shape == (self.size,), "{} != {}".format(
+                parameter_space.shape, self.size)
             self._set_parameters(parameter_space)
 
     @deprecated("set(parametername=value_array)")
@@ -404,7 +408,8 @@ class BasePopulation(object):
             p.initialize(v=lambda i: -65 + i/10.0)
         """
         for variable, value in initial_values.items():
-            logger.debug("In Population '%s', initialising %s to %s" % (self.label, variable, value))
+            logger.debug("In Population '%s', initialising %s to %s" %
+                         (self.label, variable, value))
             initial_value = LazyArray(value, shape=(self.size,), dtype=float)
             self._set_initial_value_array(variable, initial_value)
             self.initial_values[variable] = initial_value
@@ -443,8 +448,8 @@ class BasePopulation(object):
         multiple of the simulation timestep.
         """
         if variables is None:  # reset the list of things to record
-                               # note that if record(None) is called on a view of a population
-                               # recording will be reset for the entire population, not just the view
+            # note that if record(None) is called on a view of a population
+            # recording will be reset for the entire population, not just the view
             self.recorder.reset()
         else:
             logger.debug("%s.record('%s')", self.label, variables)
@@ -452,7 +457,7 @@ class BasePopulation(object):
                 self.recorder.record(variables, self.all_cells, sampling_interval)
             else:
                 self.recorder.record(variables, self._record_filter, sampling_interval)
-        if isinstance(to_file, basestring):
+        if isinstance(to_file, str):
             self.recorder.file = to_file
             self._simulator.state.write_on_end.append((self, variables, self.recorder.file))
 
@@ -493,7 +498,8 @@ class BasePopulation(object):
         numbers and strings. The contents will be written into the output data
         file as metadata.
         """
-        logger.debug("Population %s is writing %s to %s [gather=%s, clear=%s]" % (self.label, variables, io, gather, clear))
+        logger.debug("Population %s is writing %s to %s [gather=%s, clear=%s]" % (
+            self.label, variables, io, gather, clear))
         self.recorder.write(variables, io, gather, self._record_filter, clear=clear,
                             annotations=annotations)
 
@@ -564,7 +570,7 @@ class BasePopulation(object):
             else:
                 return 0
         else:
-            return numpy.nan
+            return np.nan
 
     def inject(self, current_source):
         """
@@ -579,11 +585,11 @@ class BasePopulation(object):
         """
         Save positions to file. The output format is ``index x y z``
         """
-        if isinstance(file, basestring):
+        if isinstance(file, str):
             file = recording.files.StandardTextFile(file, mode='w')
         cells = self.all_cells
-        result = numpy.empty((len(cells), 4))
-        result[:, 0] = numpy.array([self.id_to_index(id) for id in cells])
+        result = np.empty((len(cells), 4))
+        result[:, 0] = np.array([self.id_to_index(id) for id in cells])
         result[:, 1:4] = self.positions.T
         if self._simulator.state.mpi_rank == 0:
             file.write(result, {'population': self.label})
@@ -632,11 +638,13 @@ class Population(BasePopulation):
                      "You should import Population from a PyNN backend module, " \
                      "e.g. pyNN.nest or pyNN.neuron"
             raise Exception(errmsg)
-        if not isinstance(size, (int, numpy.integer)):  # also allow a single integer, for a 1D population
-            assert isinstance(size, tuple), "`size` must be an integer or a tuple of ints. You have supplied a %s" % type(size)
+        if not isinstance(size, (int, np.integer)):  # also allow a single integer, for a 1D population
+            assert isinstance(
+                size, tuple), "`size` must be an integer or a tuple of ints. You have supplied a %s" % type(size)
             # check the things inside are ints
             for e in size:
-                assert isinstance(e, int), "`size` must be an integer or a tuple of ints. Element '%s' is not an int" % str(e)
+                assert isinstance(
+                    e, int), "`size` must be an integer or a tuple of ints. Element '%s' is not an int" % str(e)
 
             assert structure is None, "If you specify `size` as a tuple you may not specify structure."
             if len(size) == 1:
@@ -648,8 +656,10 @@ class Population(BasePopulation):
                 nx, ny, nz = size
                 structure = space.Grid3D(nx / float(ny), nx / float(nz))
             else:
-                raise Exception("A maximum of 3 dimensions is allowed. What do you think this is, string theory?")
-            size = int(reduce(operator.mul, size))  # NEST doesn't like numpy.int, so to be safe we cast to Python int
+                raise Exception(
+                    "A maximum of 3 dimensions is allowed. What do you think this is, string theory?")
+            # NEST doesn't like np.int, so to be safe we cast to Python int
+            size = int(reduce(operator.mul, size))
         self.size = size
         self.label = label or 'population%d' % Population._nPop
         self._structure = structure or space.Line()
@@ -662,7 +672,8 @@ class Population(BasePopulation):
             self.celltype = cellclass(**cellparams)
             # emit deprecation warning
         else:
-            raise TypeError("cellclass must be an instance or subclass of BaseCellType, not a %s" % type(cellclass))
+            raise TypeError(
+                "cellclass must be an instance or subclass of BaseCellType, not a %s" % type(cellclass))
         self.annotations = {}
         self.recorder = self._recorder_class(self)
         # Build the arrays of cell ids
@@ -695,17 +706,19 @@ class Population(BasePopulation):
 
             >>> assert p.id_to_index(p[5]) == 5
         """
-        if not numpy.iterable(id):
+        if not np.iterable(id):
             if not self.first_id <= id <= self.last_id:
-                raise ValueError("id should be in the range [%d,%d], actually %d" % (self.first_id, self.last_id, id))
+                raise ValueError("id should be in the range [%d,%d], actually %d" % (
+                    self.first_id, self.last_id, id))
             return int(id - self.first_id)  # this assumes ids are consecutive
         else:
             if isinstance(id, PopulationView):
                 id = id.all_cells
-            id = numpy.array(id)
+            id = np.array(id)
             if (self.first_id > id.min()) or (self.last_id < id.max()):
-                raise ValueError("ids should be in the range [%d,%d], actually [%d, %d]" % (self.first_id, self.last_id, id.min(), id.max()))
-            return (id - self.first_id).astype(numpy.int)  # this assumes ids are consecutive
+                raise ValueError("ids should be in the range [%d,%d], actually [%d, %d]" % (
+                    self.first_id, self.last_id, id.min(), id.max()))
+            return (id - self.first_id).astype(np.int)  # this assumes ids are consecutive
 
     def id_to_local_index(self, id):
         """
@@ -714,7 +727,7 @@ class Population(BasePopulation):
         """
         if self._simulator.state.num_processes > 1:
             return self.local_cells.tolist().index(id)          # probably very slow
-            #return numpy.nonzero(self.local_cells == id)[0][0] # possibly faster?
+            # return np.nonzero(self.local_cells == id)[0][0] # possibly faster?
             # another idea - get global index, use idx-sum(mask_local[:idx])?
         else:
             return self.id_to_index(id)
@@ -742,7 +755,7 @@ class Population(BasePopulation):
         return self._positions
 
     def _set_positions(self, pos_array):
-        assert isinstance(pos_array, numpy.ndarray)
+        assert isinstance(pos_array, np.ndarray)
         assert pos_array.shape == (3, self.size), "%s != %s" % (pos_array.shape, (3, self.size))
         self._positions = pos_array.copy()  # take a copy in case pos_array is changed later
         self._structure = None  # explicitly setting positions destroys any previous structure
@@ -825,23 +838,27 @@ class PopulationView(BasePopulation):
         # If not, then we need to remove duplicated IDs
         if not isinstance(self.mask, slice):
             if isinstance(self.mask, list):
-                self.mask = numpy.array(self.mask)
-            if self.mask.dtype is numpy.dtype('bool'):
+                self.mask = np.array(self.mask)
+            if self.mask.dtype is np.dtype('bool'):
                 if len(self.mask) != len(self.parent):
                     raise Exception("Boolean masks should have the size of Parent Population")
-                self.mask = numpy.arange(len(self.parent))[self.mask]
-            if len(numpy.unique(self.mask)) != len(self.mask):
-                logging.warning("PopulationView can contain only once each ID, duplicated IDs are removed")
-                self.mask = numpy.unique(self.mask)
-        self.all_cells = self.parent.all_cells[self.mask]  # do we need to ensure this is ordered?
-        idx = numpy.argsort(self.all_cells)
-        self._is_sorted = numpy.all(idx == numpy.arange(len(self.all_cells)))
+                self.mask = np.arange(len(self.parent))[self.mask]
+            else:
+                if len(np.unique(self.mask)) != len(self.mask):
+                    logging.warning(
+                        "PopulationView can contain only once each ID, duplicated IDs are removed")
+                    self.mask = np.unique(self.mask)
+                self.mask.sort()  # needed by NEST. Maybe emit a warning or exception if mask is not already ordered?
+        self.all_cells = self.parent.all_cells[self.mask]
+        idx = np.argsort(self.all_cells)
+        self._is_sorted = np.all(idx == np.arange(len(self.all_cells)))
         self.size = len(self.all_cells)
         self.label = label or "view of '%s' with size %s" % (parent.label, self.size)
         self._mask_local = self.parent._mask_local[self.mask]
         self.local_cells = self.all_cells[self._mask_local]
-        self.first_id = numpy.min(self.all_cells)  # only works if we assume all_cells is sorted, otherwise could use min()
-        self.last_id = numpy.max(self.all_cells)
+        # only works if we assume all_cells is sorted, otherwise could use min()
+        self.first_id = np.min(self.all_cells)
+        self.last_id = np.max(self.all_cells)
         self.annotations = {}
         self.recorder = self.parent.recorder
         self._record_filter = self.all_cells
@@ -864,7 +881,8 @@ class PopulationView(BasePopulation):
 
     @property
     def positions(self):
-        return self.parent.positions.T[self.mask].T  # make positions N,3 instead of 3,N to avoid all this transposing?
+        # make positions N,3 instead of 3,N to avoid all this transposing?
+        return self.parent.positions.T[self.mask].T
 
     def id_to_index(self, id):
         """
@@ -873,30 +891,30 @@ class PopulationView(BasePopulation):
 
             >>> assert pv.id_to_index(pv[3]) == 3
         """
-        if not numpy.iterable(id):
+        if not np.iterable(id):
             if self._is_sorted:
                 if id not in self.all_cells:
                     raise IndexError("ID %s not present in the View" % id)
-                return numpy.searchsorted(self.all_cells, id)
+                return np.searchsorted(self.all_cells, id)
             else:
-                result = numpy.where(self.all_cells == id)[0]
+                result = np.where(self.all_cells == id)[0]
             if len(result) == 0:
                 raise IndexError("ID %s not present in the View" % id)
             else:
                 return result
         else:
             if self._is_sorted:
-                return numpy.searchsorted(self.all_cells, id)
+                return np.searchsorted(self.all_cells, id)
             else:
-                result = numpy.array([], dtype=numpy.int)
+                result = np.array([], dtype=np.int)
                 for item in id:
-                    data = numpy.where(self.all_cells == item)[0]
+                    data = np.where(self.all_cells == item)[0]
                     if len(data) == 0:
                         raise IndexError("ID %s not present in the View" % item)
                     elif len(data) > 1:
                         raise Exception("ID %s is duplicated in the View" % item)
                     else:
-                        result = numpy.append(result, data)
+                        result = np.append(result, data)
                 return result
 
     @property
@@ -918,7 +936,7 @@ class PopulationView(BasePopulation):
         Given an array of indices, return the indices in the parent population
         at the root of the tree.
         """
-        indices_in_parent = numpy.arange(self.parent.size)[self.mask][indices]
+        indices_in_parent = np.arange(self.parent.size)[self.mask][indices]
         if hasattr(self.parent, "parent"):
             return self.parent.index_in_grandparent(indices_in_parent)
         else:
@@ -937,14 +955,14 @@ class PopulationView(BasePopulation):
         else:
             if isinstance(indices, int):
                 return np.nonzero(self.mask == indices)[0][0]
-            elif isinstance(indices, numpy.ndarray):
+            elif isinstance(indices, np.ndarray):
                 # Lots of ways to do this. Some profiling is in order.
                 # - https://stackoverflow.com/questions/16992713/translate-every-element-in-numpy-array-according-to-key
                 # - https://stackoverflow.com/questions/3403973/fast-replacement-of-values-in-a-numpy-array
                 # - https://stackoverflow.com/questions/13572448/replace-values-of-a-numpy-index-array-with-values-of-a-list
                 parent_indices = self.mask  # assert mask is sorted
-                view_indices = numpy.arange(self.size)
-                index = numpy.digitize(indices, parent_indices, right=True)
+                view_indices = np.arange(self.size)
+                index = np.digitize(indices, parent_indices, right=True)
                 return view_indices[index]
             else:
                 raise ValueError("indices must be an integer or an array of integers")
@@ -962,9 +980,9 @@ class PopulationView(BasePopulation):
         # We can't use the self.mask, as different masks can select the same cells
         # (e.g. slices vs arrays), therefore we have to use self.all_cells
         if isinstance(other, PopulationView):
-            return self.parent != other.parent or not numpy.array_equal(self.all_cells, other.all_cells)
+            return self.parent != other.parent or not np.array_equal(self.all_cells, other.all_cells)
         elif isinstance(other, Population):
-            return self.parent != other or not numpy.array_equal(self.all_cells, other.all_cells)
+            return self.parent != other or not np.array_equal(self.all_cells, other.all_cells)
         else:
             return True
 
@@ -1014,7 +1032,7 @@ class Assembly(object):
         for p in populations:
             self._insert(p)
         self.label = kwargs.get('label', 'assembly%d' % Assembly._count)
-        assert isinstance(self.label, basestring), "label must be a string or unicode"
+        assert isinstance(self.label, str), "label must be a string"
         self.annotations = {}
         Assembly._count += 1
 
@@ -1028,15 +1046,17 @@ class Assembly(object):
             if not element.parent in self.populations:
                 double = False
                 for p in self.populations:
-                    data = numpy.concatenate((p.all_cells, element.all_cells))
-                    if len(numpy.unique(data)) != len(p.all_cells) + len(element.all_cells):
-                        logging.warning('Adding a PopulationView to an Assembly containing elements already present is not posible')
+                    data = np.concatenate((p.all_cells, element.all_cells))
+                    if len(np.unique(data)) != len(p.all_cells) + len(element.all_cells):
+                        logging.warning(
+                            'Adding a PopulationView to an Assembly containing elements already present is not posible')
                         double = True  # Should we automatically remove duplicated IDs ?
                         break
                 if not double:
                     self.populations.append(element)
             else:
-                logging.warning('Adding a PopulationView to an Assembly when parent Population is there is not possible')
+                logging.warning(
+                    'Adding a PopulationView to an Assembly when parent Population is there is not possible')
         elif isinstance(element, BasePopulation):
             if not element in self.populations:
                 self.populations.append(element)
@@ -1047,14 +1067,14 @@ class Assembly(object):
     def local_cells(self):
         result = self.populations[0].local_cells
         for p in self.populations[1:]:
-            result = numpy.concatenate((result, p.local_cells))
+            result = np.concatenate((result, p.local_cells))
         return result
 
     @property
     def all_cells(self):
         result = self.populations[0].all_cells
         for p in self.populations[1:]:
-            result = numpy.concatenate((result, p.all_cells))
+            result = np.concatenate((result, p.all_cells))
         return result
 
     def all(self):
@@ -1063,8 +1083,8 @@ class Assembly(object):
 
     @property
     def _is_sorted(self):
-        idx = numpy.argsort(self.all_cells)
-        return numpy.all(idx == numpy.arange(len(self.all_cells)))
+        idx = np.argsort(self.all_cells)
+        return np.all(idx == np.arange(len(self.all_cells)))
 
     @property
     def _homogeneous_synapses(self):
@@ -1106,16 +1126,16 @@ class Assembly(object):
     def _mask_local(self):
         result = self.populations[0]._mask_local
         for p in self.populations[1:]:
-            result = numpy.concatenate((result, p._mask_local))
+            result = np.concatenate((result, p._mask_local))
         return result
 
     @property
     def first_id(self):
-        return numpy.min(self.all_cells)
+        return np.min(self.all_cells)
 
     @property
     def last_id(self):
-        return numpy.max(self.all_cells)
+        return np.max(self.all_cells)
 
     def id_to_index(self, id):
         """
@@ -1126,35 +1146,35 @@ class Assembly(object):
             >>> assert p.id_to_index(p.index([1, 2, 3])) == [1, 2, 3]
         """
         all_cells = self.all_cells
-        if not numpy.iterable(id):
+        if not np.iterable(id):
             if self._is_sorted:
-                return numpy.searchsorted(all_cells, id)
+                return np.searchsorted(all_cells, id)
             else:
-                result = numpy.where(all_cells == id)[0]
+                result = np.where(all_cells == id)[0]
             if len(result) == 0:
                 raise IndexError("ID %s not present in the View" % id)
             else:
                 return result
         else:
             if self._is_sorted:
-                return numpy.searchsorted(all_cells, id)
+                return np.searchsorted(all_cells, id)
             else:
-                result = numpy.array([], dtype=numpy.int)
+                result = np.array([], dtype=np.int)
                 for item in id:
-                    data = numpy.where(all_cells == item)[0]
+                    data = np.where(all_cells == item)[0]
                     if len(data) == 0:
                         raise IndexError("ID %s not present in the Assembly" % item)
                     elif len(data) > 1:
                         raise Exception("ID %s is duplicated in the Assembly" % item)
                     else:
-                        result = numpy.append(result, data)
+                        result = np.append(result, data)
                 return result
 
     @property
     def positions(self):
         result = self.populations[0].positions
         for p in self.populations[1:]:
-            result = numpy.hstack((result, p.positions))
+            result = np.hstack((result, p.positions))
         return result
 
     @property
@@ -1180,25 +1200,28 @@ class Assembly(object):
         consisting of appropriate populations and (possibly newly created)
         population views.
         """
-        count = 0; boundaries = [0]
+        count = 0
+        boundaries = [0]
         for p in self.populations:
             count += p.size
             boundaries.append(count)
-        boundaries = numpy.array(boundaries, dtype=numpy.int)
+        boundaries = np.array(boundaries, dtype=int)
 
-        if isinstance(index, (int, numpy.integer)):  # return an ID
+        if isinstance(index, (int, np.integer)):  # return an ID
             pindex = boundaries[1:].searchsorted(index, side='right')
             return self.populations[pindex][index - boundaries[pindex]]
-        elif isinstance(index, (slice, tuple, list, numpy.ndarray)):
+        elif isinstance(index, (slice, tuple, list, np.ndarray)):
             if isinstance(index, slice) or (hasattr(index, "dtype") and index.dtype == bool):
-                indices = numpy.arange(self.size)[index]
+                indices = np.arange(self.size)[index]
             else:
-                indices = numpy.array(index)
+                indices = np.array(index)
             pindices = boundaries[1:].searchsorted(indices, side='right')
-            views = [self.populations[i][indices[pindices == i] - boundaries[i]] for i in numpy.unique(pindices)]
+            views = [self.populations[i][indices[pindices == i] - boundaries[i]]
+                     for i in np.unique(pindices)]
             return self.__class__(*views)
         else:
-            raise TypeError("indices must be integers, slices, lists, arrays, not %s" % type(index).__name__)
+            raise TypeError("indices must be integers, slices, lists, arrays, not %s" %
+                            type(index).__name__)
 
     def __add__(self, other):
         """
@@ -1238,7 +1261,7 @@ class Assembly(object):
         assert isinstance(n, int)
         if not rng:
             rng = random.NumpyRNG()
-        indices = rng.permutation(numpy.arange(len(self), dtype=numpy.int))[0:n]
+        indices = rng.permutation(np.arange(len(self), dtype=np.int))[0:n]
         logger.debug("The %d cells recorded have indices %s" % (n, indices))
         logger.debug("%s.sample(%s)", self.label, n)
         return self[indices]
@@ -1256,7 +1279,7 @@ class Assembly(object):
         Get the values of the given parameters for every local cell in the
         Assembly, or, if gather=True, for all cells in the Assembly.
         """
-        if isinstance(parameter_names, basestring):
+        if isinstance(parameter_names, str):
             parameter_names = (parameter_names,)
             return_list = False
         else:
@@ -1268,7 +1291,7 @@ class Assembly(object):
             for name, arr in zip(parameter_names, population_values):
                 parameters[name].append(arr)
         for name, value_list in parameters.items():
-            parameters[name] = numpy.hstack(value_list)
+            parameters[name] = np.hstack(value_list)
             if simplify:
                 parameters[name] = simplify_parameter_array(parameters[name])
         values = [parameters[name] for name in parameter_names]
@@ -1336,11 +1359,11 @@ class Assembly(object):
         """
         Save positions to file. The output format is id x y z
         """
-        if isinstance(file, basestring):
+        if isinstance(file, str):
             file = files.StandardTextFile(file, mode='w')
         cells = self.all_cells
-        result = numpy.empty((len(cells), 4))
-        result[:, 0] = numpy.array([self.id_to_index(id) for id in cells])
+        result = np.empty((len(cells), 4))
+        result[:, 0] = np.array([self.id_to_index(id) for id in cells])
         result[:, 1:4] = self.positions.T
         if self._simulator.state.mpi_rank == 0:
             file.write(result, {'assembly': self.label})
@@ -1376,7 +1399,7 @@ class Assembly(object):
         for block, p in zip(blocks, self.populations):
             for segment in block.segments:
                 for signal_array in segment.analogsignals:
-                    signal_array.channel_index.channel_ids += offset
+                    signal_array.array_annotations["channel_index"] += offset
             offset += p.size
         for i, block in enumerate(blocks):
             logger.debug("%d: %s", i, block.name)
@@ -1414,14 +1437,15 @@ class Assembly(object):
         if self._simulator.state.mpi_rank == 0 or not gather:  # should maybe use allgather, and get the numbers on all nodes
             return float(total_spikes) / len(spike_counts)
         else:
-            return numpy.nan
+            return np.nan
 
     def get_spike_counts(self, gather=True):
         """
         Returns the number of spikes for each neuron.
         """
         try:
-            spike_counts = self.populations[0].recorder.count('spikes', gather, self.populations[0]._record_filter)
+            spike_counts = self.populations[0].recorder.count(
+                'spikes', gather, self.populations[0]._record_filter)
         except errors.NothingToWriteError:
             spike_counts = {}
         for p in self.populations[1:]:
@@ -1450,12 +1474,12 @@ class Assembly(object):
 
         If `clear` is True, recorded data will be deleted from the `Population`.
         """
-        if isinstance(io, basestring):
+        if isinstance(io, str):
             io = recording.get_io(io)
         if gather is False and self._simulator.state.num_processes > 1:
             io.filename += '.%d' % self._simulator.state.mpi_rank
         logger.debug("Recorder is writing '%s' to file '%s' with gather=%s" % (
-                                               variables, io.filename, gather))
+            variables, io.filename, gather))
         data = self.get_data(variables, gather, clear, annotations)
         if self._simulator.state.mpi_rank == 0 or gather is False:
             logger.debug("Writing data to file %s" % io)
@@ -1502,7 +1526,7 @@ class Assembly(object):
         """
         Get the values of the given annotations for each population in the Assembly.
         """
-        if isinstance(annotation_keys, basestring):
+        if isinstance(annotation_keys, str):
             annotation_keys = (annotation_keys,)
         annotations = defaultdict(list)
 
@@ -1511,9 +1535,9 @@ class Assembly(object):
             for p in self.populations:
                 annotation = p.annotations[key]
                 annotations[key].append(annotation)
-                is_array_annotation = isinstance(annotation, numpy.ndarray)
+                is_array_annotation = isinstance(annotation, np.ndarray)
             if is_array_annotation:
-                annotations[key] = numpy.hstack(annotations[key])
+                annotations[key] = np.hstack(annotations[key])
             if simplify:
-                annotations[key] = simplify_parameter_array(numpy.array(annotations[key]))
+                annotations[key] = simplify_parameter_array(np.array(annotations[key]))
         return annotations
