@@ -1,12 +1,12 @@
 """
 Assorted utility classes and functions.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 """
 
 import warnings
-import numpy
+import numpy as np
 
 
 def is_listlike(obj):
@@ -18,28 +18,10 @@ def is_listlike(obj):
     """
     return (
         isinstance(obj, (list, tuple, set))
-        or (isinstance(obj, numpy.ndarray) and obj.ndim > 0)
+        or (isinstance(obj, np.ndarray) and obj.ndim > 0)
     )
 
 
-def iteritems(obj):
-    """Handle the disappearance of `dict.iteritems` in Python 3"""
-    try:
-        itr = obj.iteritems()  # Python 2
-    except AttributeError:
-        itr = obj.items()
-    return itr
-
-
-def itervalues(obj):
-    """Handle the disappearance of `dict.itervalues` in Python 3"""
-    try:
-        itr = obj.itervalues()  # Python 2
-    except AttributeError:
-        itr = obj.values()
-    return itr
-
-
 class deprecated(object):
     """
     Decorator to mark functions/methods as deprecated. Emits a warning when
