@@ -4,7 +4,7 @@ Mock implementation of the PyNN API, for testing and documentation purposes.
 This simulator implements the PyNN API, but generates random data rather than
 really running simulations.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 """
 
@@ -50,6 +50,7 @@ def end(compatible_output=True):
     simulator.state.write_on_end = []
     # should have common implementation of end()
 
+
 run, run_until = common.build_run(simulator)
 run_for = run
 
@@ -58,7 +59,7 @@ reset = common.build_reset(simulator)
 initialize = common.initialize
 
 get_current_time, get_time_step, get_min_delay, get_max_delay, \
-                    num_processes, rank = common.build_state_queries(simulator)
+    num_processes, rank = common.build_state_queries(simulator)
 
 create = common.build_create(Population)
 
@@ -67,6 +68,8 @@ connect = common.build_connect(Projection, FixedProbabilityConnector, StaticSyna
 
 record = common.build_record(simulator)
 
-record_v = lambda source, filename: record(['v'], source, filename)
 
-record_gsyn = lambda source, filename: record(['gsyn_exc', 'gsyn_inh'], source, filename)
+def record_v(source, filename): return record(['v'], source, filename)
+
+
+def record_gsyn(source, filename): return record(['gsyn_exc', 'gsyn_inh'], source, filename)
