@@ -2,7 +2,7 @@
 Base classes for cell and synapse models, whether "standard" (cross-simulator)
 or "native" (restricted to an individual simulator).
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 """
 
@@ -27,7 +27,8 @@ class BaseModelType(object):
             self.parameter_space.update(**parameters)
 
     def __repr__(self):
-        return "%s(<parameters>)" % self.__class__.__name__  # should really include the parameters explicitly, to be unambiguous
+        # should really include the parameters explicitly, to be unambiguous
+        return "%s(<parameters>)" % self.__class__.__name__
 
     @classmethod
     def has_parameter(cls, name):
@@ -85,7 +86,8 @@ class BaseCurrentSource(BaseModelType):
 class BaseSynapseType(BaseModelType):
     """Base class for synapse model classes."""
 
-    connection_type = None  # override to specify a non-standard connection type (i.e. GapJunctions)
+    # override to specify a non-standard connection type (i.e. GapJunctions)
+    connection_type = None
     has_presynaptic_components = False  # override for synapses that include an active presynaptic components
 
     def __init__(self, **parameters):
