@@ -1,7 +1,8 @@
+# -*- coding: utf-8 -*-
 """
 Definition of NativeCellType class for NEST.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 """
 
@@ -40,7 +41,8 @@ def get_defaults(model_name):
             if isinstance(value, valid_types):
                 default_params[name] = conversion.make_pynn_compatible(value)
             else:
-                warnings.warn("Ignoring parameter '%s' since PyNN does not support %s" % (name, type(value)))
+                warnings.warn("Ignoring parameter '%s' since PyNN does not support %s" %
+                              (name, type(value)))
     return default_params, default_initial_values
 
 
@@ -49,7 +51,10 @@ def get_receptor_types(model_name):
 
 
 def get_recordables(model_name):
-    return [sl.name for sl in nest.GetDefaults(model_name).get("recordables", [])]
+    try:
+        return [name for name in nest.GetDefaults(model_name, "recordables")]
+    except nest.NESTError as err:
+        return []
 
 
 def native_cell_type(model_name):
@@ -60,7 +65,7 @@ def native_cell_type(model_name):
     default_parameters, default_initial_values = get_defaults(model_name)
     receptor_types = get_receptor_types(model_name)
     recordable = get_recordables(model_name) + ['spikes']
-    element_type = nest.GetDefaults(model_name, 'element_type').name
+    element_type = nest.GetDefaults(model_name, 'element_type')
     return type(model_name,
                 (NativeCellType,),
                 {'nest_model': model_name,
