@@ -2,12 +2,12 @@
 """
 NEST v2 implementation of the PyNN API.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
 """
 
-import numpy
+import numpy as np
 import nest
 import logging
 from pyNN import common, errors
@@ -29,28 +29,31 @@ class PopulationMixin(object):
         """
         parameter_space should contain native parameters
         """
-        param_dict = _build_params(parameter_space, numpy.where(self._mask_local)[0])
-        ids = self.local_cells.tolist()
+        param_dict = _build_params(parameter_space, np.where(self._mask_local)[0])
         if hasattr(self.celltype, "uses_parrot") and self.celltype.uses_parrot:
-            ids = [id.source for id in ids]
-        nest.SetStatus(ids, param_dict)
+            ids = self.node_collection_source[self._mask_local]
+        else:
+            ids = self.node_collection[self._mask_local]
+        simulator.state.set_status(ids, param_dict)
 
     def _get_parameters(self, *names):
         """
         return a ParameterSpace containing native parameters
         """
-        ids = self.local_cells.tolist()
         if hasattr(self.celltype, "uses_parrot") and self.celltype.uses_parrot:
-            ids = [id.source for id in ids]
+            ids = self.node_collection_source[self._mask_local]
+        else:
+            ids = self.node_collection[self._mask_local]
 
         if "spike_times" in names:
-            parameter_dict = {"spike_times": [Sequence(value) for value in nest.GetStatus(ids, names)]}
+            parameter_dict = {"spike_times": [Sequence(value)
+                                              for value in nest.GetStatus(ids, names)]}
         else:
             parameter_dict = {}
             for name in names:  # one name at a time, since some parameter values may be tuples
-                val = numpy.array(nest.GetStatus(ids, name))
+                val = np.array(nest.GetStatus(ids, name))
                 if isinstance(val[0], tuple) or len(val.shape) == 2:
-                    val = numpy.array([ArrayParameter(v) for v in val])
+                    val = np.array([ArrayParameter(v) for v in val])
                     val = LazyArray(simplify(val), shape=(self.local_size,), dtype=ArrayParameter)
                     parameter_dict[name] = val
                 else:
@@ -58,17 +61,41 @@ class PopulationMixin(object):
         ps = ParameterSpace(parameter_dict, shape=(self.local_size,))
         return ps
 
+    @property
+    def local_node_collection(self):
+        return self.node_collection[self._mask_local]
+
 
 class Assembly(common.Assembly):
     __doc__ = common.Assembly.__doc__
     _simulator = simulator
 
+    @property
+    def local_node_collection(self):
+        result = self.populations[0].local_node_collection
+        for p in self.populations[1:]:
+            result += p.local_node_collection
+        return result
+
+    @property
+    def node_collection(self):
+        return sum((p.node_collection for p in self.populations[1:]),
+                   start=self.populations[0].node_collection)
+
 
 class PopulationView(common.PopulationView, PopulationMixin):
     __doc__ = common.PopulationView.__doc__
     _simulator = simulator
     _assembly_class = Assembly
 
+    @property
+    def node_collection(self):
+        return self.parent.node_collection[self.mask]
+
+    @property
+    def node_collection_source(self):
+        return self.parent.node_collection_source[self.mask]
+
 
 def _build_params(parameter_space, mask_local, size=None, extra_parameters=None):
     """
@@ -111,7 +138,8 @@ class Population(common.Population, PopulationMixin):
                  initial_values={}, label=None):
         __doc__ = common.Population.__doc__
         self._deferred_parrot_connections = False
-        super(Population, self).__init__(size, cellclass, cellparams, structure, initial_values, label)
+        super(Population, self).__init__(size, cellclass,
+                                         cellparams, structure, initial_values, label)
         self._simulator.state.populations.append(self)
 
     def _create_cells(self):
@@ -132,7 +160,7 @@ class Population(common.Population, PopulationMixin):
                                    None,
                                    size=self.size)
         try:
-            self.all_cells = nest.Create(nest_model, self.size, params=params)
+            self.node_collection = nest.Create(nest_model, self.size, params=params)
         except nest.kernel.NESTError as err:
             if "UnknownModelName" in err.args[0] and "cond" in err.args[0]:
                 raise errors.InvalidModelError("%s Have you compiled NEST with the GSL (Gnu Scientific Library)?" % err)
@@ -141,26 +169,38 @@ class Population(common.Population, PopulationMixin):
             raise  # errors.InvalidModelError(err)
         # create parrot neurons if necessary
         if hasattr(self.celltype, "uses_parrot") and self.celltype.uses_parrot:
-            self.all_cells_source = numpy.array(self.all_cells)        # we put the parrots into all_cells, since this will
+            self.node_collection_source = self.node_collection          # we put the parrots into all_cells, since this will
             parrot_model = simulator.state.spike_precision == "off_grid" and "parrot_neuron_ps" or "parrot_neuron"
-            self.all_cells = nest.Create(parrot_model, self.size)      # be used for connections and recording. all_cells_source
-                                                                       # should be used for setting parameters
+            self.node_collection = nest.Create(parrot_model, self.size) # be used for connections and recording. all_cells_source
+                                                                        # should be used for setting parameters
             self._deferred_parrot_connections = True
             # connecting up the parrot neurons is deferred until we know the value of min_delay
             # which could be 'auto' at this point.
-        self._mask_local = numpy.array(nest.GetStatus(self.all_cells, 'local'))
-        self.all_cells = numpy.array([simulator.ID(gid) for gid in self.all_cells], simulator.ID)
+        if self.node_collection.local is True:
+            self._mask_local = np.array([True])
+        else:
+            self._mask_local = np.array(self.node_collection.local)
+        self.all_cells = np.array([simulator.ID(gid) for gid in self.node_collection.tolist()], simulator.ID)
         for gid in self.all_cells:
             gid.parent = self
+            gid.node_collection = nest.NodeCollection([int(gid)])
         if hasattr(self.celltype, "uses_parrot") and self.celltype.uses_parrot:
-            for gid, source in zip(self.all_cells, self.all_cells_source):
+            for gid, source in zip(self.all_cells, self.node_collection_source.tolist()):
                 gid.source = source
 
     def _connect_parrot_neurons(self):
-        nest.Connect(self.all_cells_source, numpy.array(self.all_cells, int), 'one_to_one',
+        nest.Connect(self.node_collection_source, self.node_collection, 'one_to_one',
                      syn_spec={'delay': simulator.state.min_delay})
         self._deferred_parrot_connections = False
 
+    def _reset(self):
+        # adjust parameters that represent absolute times for the time offset after reset
+        if hasattr(self.celltype, "uses_parrot") and self.celltype.uses_parrot:
+            for name in self.celltype.get_native_names():
+                if name in ("start", "stop", "spike_times"):
+                    value = self.celltype.native_parameters[name]
+                    self._simulator.set_status(self.node_collection, name, value)
+
     def _set_initial_value_array(self, variable, value):
         variable = VARIABLE_MAP.get(variable, variable)
         if isinstance(value.base_value, RandomDistribution) and value.base_value.rng.parallel_safe:
@@ -168,7 +208,10 @@ class Population(common.Population, PopulationMixin):
         else:
             local_values = value._partially_evaluate(self._mask_local, simplify=True)
         try:
-            nest.SetStatus(self.local_cells.tolist(), variable, local_values)
+            if self._mask_local.dtype == bool and self._mask_local.size == 1 and self._mask_local[0]:
+                simulator.state.set_status(self.node_collection, variable, local_values)
+            else:
+                simulator.state.set_status(self.node_collection[self._mask_local], variable, local_values)
         except nest.kernel.NESTError as e:
             if "Unused dictionary items" in e.args[0]:
                 logger.warning("NEST does not allow setting an initial value for %s" % variable)
