@@ -8,12 +8,12 @@ Classes:
     ACSource           -- a sine modulated current.
 
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
 """
 
-import numpy
+import numpy as np
 import nest
 from pyNN.standardmodels import electrodes, build_translations, StandardCurrentSource
 from pyNN.common import Population, PopulationView, Assembly
@@ -37,9 +37,9 @@ class NestStandardCurrentSource(NestCurrentSource, StandardCurrentSource):
             if id.local and not id.celltype.injectable:
                 raise TypeError("Can't inject current into a spike source.")
         if isinstance(cells, (Population, PopulationView, Assembly)):
-            self.cell_list = [cell for cell in cells]
+            self.cell_list = cells.node_collection
         else:
-            self.cell_list = cells
+            self.cell_list = nest.NodeCollection(sorted(cells))
         nest.Connect(self._device, self.cell_list, syn_spec={"delay": state.min_delay})
 
     def _phase_correction(self, start, freq, phase):
@@ -48,7 +48,7 @@ class NestStandardCurrentSource(NestCurrentSource, StandardCurrentSource):
         Tweaks the value of phase supplied to NEST ACSource
         so as to remain consistent with other simulators
         """
-        phase_fix = ( (phase*numpy.pi/180) - (2*numpy.pi*freq*start/1000)) * 180/numpy.pi
+        phase_fix = ((phase*np.pi/180) - (2*np.pi*freq*start/1000)) * 180/np.pi
         phase_fix.shape = (1)
         phase_fix = phase_fix.evaluate()[0]
         nest.SetStatus(self._device, {'phase': phase_fix})
@@ -59,8 +59,8 @@ class NestStandardCurrentSource(NestCurrentSource, StandardCurrentSource):
         """
         corrected = value - self.min_delay
         # set negative times to zero
-        if isinstance(value, numpy.ndarray):
-            corrected = numpy.where(corrected > 0, corrected, 0.0)
+        if isinstance(value, np.ndarray):
+            corrected = np.where(corrected > 0, corrected, 0.0)
         else:
             corrected = max(corrected, 0.0)
         return corrected
@@ -71,36 +71,36 @@ class NestStandardCurrentSource(NestCurrentSource, StandardCurrentSource):
         Tweaks the value of phase supplied to NEST ACSource
         so as to remain consistent with other simulators
         """
-        phase_fix = ( (phase*numpy.pi/180) - (2*numpy.pi*freq*start/1000)) * 180/numpy.pi
+        phase_fix = ((phase*np.pi/180) - (2*np.pi*freq*start/1000)) * 180/np.pi
         phase_fix.shape = (1)
         phase_fix = phase_fix.evaluate()[0]
         nest.SetStatus(self._device, {'phase': phase_fix})
 
     def _check_step_times(self, times, amplitudes, resolution):
         # ensure that all time stamps are non-negative
-        if numpy.min(times) < 0:
+        if np.min(times) < 0:
             raise ValueError("Step current cannot accept negative timestamps.")
         # ensure that times provided are of strictly increasing magnitudes
-        if len(times)> 1 and numpy.min(numpy.diff(times)) <= 0:
+        if len(times) > 1 and np.min(np.diff(times)) <= 0:
             raise ValueError("Step current timestamps should be monotonically increasing.")
         # NEST specific: subtract min_delay from times (set to 0.0, if result is negative)
         times = self._delay_correction(times)
         # find the last element <= dt (we find >dt and then go one element back)
         # this corresponds to the first timestamp that can be used by NEST for current injection
-        ctr = numpy.searchsorted(times, resolution, side="right") - 1
+        ctr = np.searchsorted(times, resolution, side="right") - 1
         if ctr >= 0:
             times[ctr] = resolution
             times = times[ctr:]
             amplitudes = amplitudes[ctr:]
         # map timestamps to actual simulation time instants based on specified dt
-        #for ind in range(len(times)):
+        # for ind in range(len(times)):
         #    times[ind] = self._round_timestamp(times[ind], resolution)
-        times = self._round_timestamp(times,resolution)
+        times = self._round_timestamp(times, resolution)
         # remove duplicate timestamps, and corresponding amplitudes, after mapping
-        step_times, step_indices = numpy.unique(times[::-1], return_index=True)
+        step_times, step_indices = np.unique(times[::-1], return_index=True)
         step_times = step_times.tolist()
         step_indices = len(times)-step_indices-1
-        step_amplitudes = amplitudes[step_indices] #[amplitudes[i] for i in step_indices]
+        step_amplitudes = amplitudes[step_indices]  # [amplitudes[i] for i in step_indices]
         return step_times, step_amplitudes
 
     def set_native_parameters(self, parameters):
@@ -111,7 +111,8 @@ class NestStandardCurrentSource(NestCurrentSource, StandardCurrentSource):
                 step_times = parameters["amplitude_times"].value
                 step_amplitudes = parameters["amplitude_values"].value
 
-                step_times, step_amplitudes = self._check_step_times(step_times, step_amplitudes, self.timestep)
+                step_times, step_amplitudes = self._check_step_times(
+                    step_times, step_amplitudes, self.timestep)
                 parameters["amplitude_times"].value = step_times
                 parameters["amplitude_values"].value = step_amplitudes
                 nest.SetStatus(self._device, {key: step_amplitudes,
