@@ -2,7 +2,7 @@
 """
 nrnpython implementation of the PyNN API.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
 """
@@ -96,7 +96,8 @@ def end(compatible_output=True):
         io = get_io(filename)
         population.write_data(io, variables)
     simulator.state.write_on_end = []
-    #simulator.state.finalize()
+    # simulator.state.finalize()
+
 
 run, run_until = common.build_run(simulator)
 run_for = run
@@ -110,7 +111,7 @@ initialize = common.initialize
 # ==============================================================================
 
 get_current_time, get_time_step, get_min_delay, get_max_delay, \
-            num_processes, rank = common.build_state_queries(simulator)
+    num_processes, rank = common.build_state_queries(simulator)
 
 
 # ==============================================================================
@@ -125,8 +126,10 @@ set = common.set
 
 record = common.build_record(simulator)
 
-record_v = lambda source, filename: record(['v'], source, filename)
 
-record_gsyn = lambda source, filename: record(['gsyn_exc', 'gsyn_inh'], source, filename)
+def record_v(source, filename): return record(['v'], source, filename)
+
+
+def record_gsyn(source, filename): return record(['gsyn_exc', 'gsyn_inh'], source, filename)
 
 # ==============================================================================
