@@ -2,14 +2,15 @@
 """
 Definition of cell classes for the neuron module.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
 """
 
 import logging
 from math import pi
-import numpy
+from functools import reduce
+import numpy as np
 from neuron import h, nrn, hclass
 
 from pyNN import errors
@@ -17,11 +18,6 @@ from pyNN.models import BaseCellType
 from .recording import recordable_pattern
 from .simulator import state
 
-try:
-    reduce
-except NameError:
-    from functools import reduce
-
 logger = logging.getLogger("PyNN")
 
 
@@ -121,7 +117,8 @@ class SingleCompartmentNeuron(BaseSingleCompartmentNeuron):
         self.syn_shape = syn_shape
 
         # insert synapses
-        assert syn_type in ('current', 'conductance'), "syn_type must be either 'current' or 'conductance'. Actual value is %s" % syn_type
+        assert syn_type in (
+            'current', 'conductance'), "syn_type must be either 'current' or 'conductance'. Actual value is %s" % syn_type
         assert syn_shape in ('alpha', 'exp'), "syn_type must be either 'alpha' or 'exp'"
         synapse_model = self.synapse_models[syn_type][syn_shape]
         self.esyn = synapse_model(0.5, sec=self)
@@ -174,8 +171,9 @@ class LeakySingleCompartmentNeuron(SingleCompartmentNeuron):
         self.v_init = v_rest  # default value
 
     def __set_tau_m(self, value):
-        #print("setting tau_m to", value, "cm =", self.seg.cm))
-        self.seg.pas.g = 1e-3 * self.seg.cm / value  # cm(nF)/tau_m(ms) = G(uS) = 1e-6G(S). Divide by area (1e-3) to get factor of 1e-3
+        # print("setting tau_m to", value, "cm =", self.seg.cm))
+        # cm(nF)/tau_m(ms) = G(uS) = 1e-6G(S). Divide by area (1e-3) to get factor of 1e-3
+        self.seg.pas.g = 1e-3 * self.seg.cm / value
 
     def __get_tau_m(self):
         #print("tau_m = ", 1e-3*self.seg.cm/self.seg.pas.g, "cm = ", self.seg.cm)
@@ -194,9 +192,9 @@ class LeakySingleCompartmentNeuron(SingleCompartmentNeuron):
     v_rest = _new_property('seg.pas', 'e')
     tau_m = property(fget=__get_tau_m, fset=__set_tau_m)
     c_m = property(fget=__get_cm, fset=__set_cm)  # if the property were called 'cm'
-                                                    # it would never get accessed as the
-                                                    # built-in Section.cm would always
-                                                    # be used first
+    # it would never get accessed as the
+    # built-in Section.cm would always
+    # be used first
 
 
 class StandardIF(LeakySingleCompartmentNeuron):
@@ -257,7 +255,7 @@ class BretteGerstnerIF(LeakySingleCompartmentNeuron):
     t_refrac = _new_property('adexp', 'trefrac')
     B = _new_property('adexp', 'b')
     A = _new_property('adexp', 'a')
-    ## using 'A' because for some reason, cell.a gives the error "NameError: a, the mechanism does not exist at PySec_170bb70(0.5)"
+    # using 'A' because for some reason, cell.a gives the error "NameError: a, the mechanism does not exist at PySec_170bb70(0.5)"
     tau_w = _new_property('adexp', 'tauw')
     delta = _new_property('adexp', 'delta')
 
@@ -270,7 +268,8 @@ class BretteGerstnerIF(LeakySingleCompartmentNeuron):
     v_spike = property(fget=__get_v_spike, fset=__set_v_spike)
 
     def __set_tau_m(self, value):
-        self.seg.pas.g = 1e-3 * self.seg.cm / value  # cm(nF)/tau_m(ms) = G(uS) = 1e-6G(S). Divide by area (1e-3) to get factor of 1e-3
+        # cm(nF)/tau_m(ms) = G(uS) = 1e-6G(S). Divide by area (1e-3) to get factor of 1e-3
+        self.seg.pas.g = 1e-3 * self.seg.cm / value
         self.adexp.GL = self.seg.pas.g * self.area() * 1e-2  # S/cm2 to uS
 
     def __get_tau_m(self):
@@ -324,7 +323,7 @@ class Izhikevich_(BaseSingleCompartmentNeuron):
     b = _new_property('izh', 'b')
     c = _new_property('izh', 'c')
     d = _new_property('izh', 'd')
-    ## using 'a_' because for some reason, cell.a gives the error "NameError: a, the mechanism does not exist at PySec_170bb70(0.5)"
+    # using 'a_' because for some reason, cell.a gives the error "NameError: a, the mechanism does not exist at PySec_170bb70(0.5)"
 
     def get_threshold(self):
         return self.izh.vthresh
@@ -475,7 +474,6 @@ class GIFNeuron(LeakySingleCompartmentNeuron):
     def __set_a_eta(self, value):
         self.gif_fun.a_eta1, self.gif_fun.a_eta2, self.gif_fun.a_eta3 = value.value
 
-
     def __get_a_eta(self):
         return self.gif_fun.a_eta1, self.gif_fun.a_eta2, self.gif_fun.a_eta3
 
@@ -484,7 +482,6 @@ class GIFNeuron(LeakySingleCompartmentNeuron):
     def __set_tau_gamma(self, value):
         self.gif_fun.tau_gamma1, self.gif_fun.tau_gamma2, self.gif_fun.tau_gamma3 = value.value
 
-
     def __get_tau_gamma(self):
         return self.gif_fun.tau_gamma1, self.gif_fun.tau_gamma2, self.gif_fun.tau_gamma3
 
@@ -529,7 +526,11 @@ class RandomSpikeSource(hclass(h.NetStimFD)):
         self.rec = h.NetCon(self, None)
         self.switch = h.NetCon(None, self)
         self.source_section = None
-        self.seed(state.mpi_rank + state.native_rng_baseseed)  # should allow user to set specific seeds somewhere, e.g. in setup()
+        # should allow user to set specific seeds somewhere, e.g. in setup()
+        self.seed(state.mpi_rank + state.native_rng_baseseed)
+
+    def __new__(cls, *arg, **kwargs):
+        return super().__new__(cls, *arg, **kwargs)
 
     def _set_interval(self, value):
         self.switch.weight[0] = -1
@@ -558,6 +559,9 @@ class RandomPoissonRefractorySpikeSource(hclass(h.PoissonStimRefractory)):
         self.source_section = None
         self.seed(state.mpi_rank + state.native_rng_baseseed)
 
+    def __new__(cls, *arg, **kwargs):
+        return super().__new__(cls, *arg, **kwargs)
+
 
 class RandomGammaSpikeSource(hclass(h.GammaStim)):
 
@@ -575,6 +579,9 @@ class RandomGammaSpikeSource(hclass(h.GammaStim)):
         self.source_section = None
         self.seed(state.mpi_rank + state.native_rng_baseseed)
 
+    def __new__(cls, *arg, **kwargs):
+        return super().__new__(cls, *arg, **kwargs)
+
 
 class VectorSpikeSource(hclass(h.VecStim)):
 
@@ -586,7 +593,10 @@ class VectorSpikeSource(hclass(h.VecStim)):
         self.source = self
         self.source_section = None
         self.rec = None
-        self._recorded_spikes = numpy.array([])
+        self._recorded_spikes = np.array([])
+
+    def __new__(cls, *arg, **kwargs):
+        return super().__new__(cls, *arg, **kwargs)
 
     def _set_spike_times(self, spike_times):
         # spike_times should be a Sequence object
@@ -594,11 +604,12 @@ class VectorSpikeSource(hclass(h.VecStim)):
             self._spike_times = h.Vector(spike_times.value)
         except (RuntimeError, AttributeError):
             raise errors.InvalidParameterValueError("spike_times must be an array of floats")
-        if numpy.any(spike_times.value[:-1] > spike_times.value[1:]):
-            raise errors.InvalidParameterValueError("Spike times given to SpikeSourceArray must be in increasing order")
+        if np.any(spike_times.value[:-1] > spike_times.value[1:]):
+            raise errors.InvalidParameterValueError(
+                "Spike times given to SpikeSourceArray must be in increasing order")
         self.play(self._spike_times)
         if self.recording:
-            self._recorded_spikes = numpy.hstack((self._recorded_spikes, spike_times.value))
+            self._recorded_spikes = np.hstack((self._recorded_spikes, spike_times.value))
 
     def _get_spike_times(self):
         return self._spike_times
@@ -615,7 +626,7 @@ class VectorSpikeSource(hclass(h.VecStim)):
         self._recording = value
         if value:
             # when we turn recording on, the cell may already have had its spike times assigned
-            self._recorded_spikes = numpy.hstack((self._recorded_spikes, self.spike_times))
+            self._recorded_spikes = np.hstack((self._recorded_spikes, self.spike_times))
 
     def get_recorded_spike_times(self):
         return self._recorded_spikes
@@ -645,12 +656,14 @@ class ArtificialCell(object):
 
     def _set_tau(self, value):
         self.source.tau = value
+
     def _get_tau(self):
         return self.source.tau
     tau = property(fget=_get_tau, fset=_set_tau)
 
     def _set_refrac(self, value):
         self.source.refrac = value
+
     def _get_refrac(self):
         return self.source.refrac
     refrac = property(fget=_get_refrac, fset=_set_refrac)
@@ -675,7 +688,7 @@ class IntFire2(NativeCellType):
     default_parameters = {'taum': 10.0, 'taus': 20.0, 'ib': 0.0}
     default_initial_values = {'m': 0.0, 'i': 0.0}
     recordable = ['m', 'i']
-    units = {'m' : 'dimensionless', 'i': 'dimensionless'}
+    units = {'m': 'dimensionless', 'i': 'dimensionless'}
     receptor_types = ['default']
     model = ArtificialCell
     extra_parameters = {"mechanism_name": "IntFire2"}
@@ -690,10 +703,10 @@ class IntFire4(NativeCellType):
     }
     default_initial_values = {'m': 0.0, 'e': 0.0, 'i1': 0.0, 'i2': 0.0}
     recordable = ['e', 'i1', 'i2', 'm']
-    units = {'e' : 'dimensionless',
-             'i1' : 'dimensionless',
-             'i2' : 'dimensionless',
-             'm' : 'dimensionless'}
+    units = {'e': 'dimensionless',
+             'i1': 'dimensionless',
+             'i2': 'dimensionless',
+             'm': 'dimensionless'}
     receptor_types = ['default']
     model = ArtificialCell
     extra_parameters = {"mechanism_name": "IntFire4"}
