@@ -7,13 +7,13 @@ Classes:
     NoisyCurrentSource -- a Gaussian whitish noise current.
     ACSource           -- a sine modulated current.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
 """
 
 from neuron import h
-import numpy
+import numpy as np
 from pyNN.standardmodels import electrodes, build_translations, StandardCurrentSource
 from pyNN.parameters import ParameterSpace, Sequence
 from pyNN.neuron import simulator
@@ -93,8 +93,8 @@ class NeuronCurrentSource(StandardCurrentSource):
         if not (times >= 0.0).all():
             raise ValueError("Step current cannot accept negative timestamps.")
         # ensure that times provided are of strictly increasing magnitudes
-        dt_times = numpy.diff(times)
-        if not all(dt_times>0.0):
+        dt_times = np.diff(times)
+        if not all(dt_times > 0.0):
             raise ValueError("Step current timestamps should be monotonically increasing.")
         # map timestamps to actual simulation time instants based on specified dt
         for ind in range(len(times)):
@@ -113,10 +113,11 @@ class NeuronCurrentSource(StandardCurrentSource):
     def set_native_parameters(self, parameters):
         parameters.evaluate(simplify=True)
         for name, value in parameters.items():
-            if name == "amplitudes": # key used only by StepCurrentSource
+            if name == "amplitudes":  # key used only by StepCurrentSource
                 step_times = parameters["times"].value
                 step_amplitudes = parameters["amplitudes"].value
-                step_times, step_amplitudes = self._check_step_times(step_times, step_amplitudes, simulator.state.dt)
+                step_times, step_amplitudes = self._check_step_times(
+                    step_times, step_amplitudes, simulator.state.dt)
                 parameters["times"].value = step_times
                 parameters["amplitudes"].value = step_amplitudes
             if isinstance(value, Sequence):  # this shouldn't be necessary, but seems to prevent a segfault
@@ -151,9 +152,9 @@ class NeuronCurrentSource(StandardCurrentSource):
         # This requires removing the first element from the current Vector
         # as NEURON computes the currents one time step later. The vector length
         # is compensated by repeating the last recorded value of current.
-        t_arr = numpy.array(self.record_times)
-        i_arr = numpy.array(self.itrace)[1:]
-        i_arr = numpy.append(i_arr, i_arr[-1])
+        t_arr = np.array(self.record_times)
+        i_arr = np.array(self.itrace)[1:]
+        i_arr = np.append(i_arr, i_arr[-1])
         return (t_arr, i_arr)
 
 
@@ -211,9 +212,9 @@ class ACSource(NeuronCurrentSource, electrodes.ACSource):
         # Not efficient at all... Is there a way to have those vectors computed on the fly ?
         # Otherwise should have a buffer mechanism
         temp_num_t = int(round(((self.stop + simulator.state.dt) - self.start) / simulator.state.dt))
-        tmp = simulator.state.dt * numpy.arange(temp_num_t)
+        tmp = simulator.state.dt * np.arange(temp_num_t)
         self.times = tmp + self.start
-        self.amplitudes = self.offset + self.amplitude * numpy.sin(tmp * 2 * numpy.pi * self.frequency / 1000. + 2 * numpy.pi * self.phase / 360)
+        self.amplitudes = self.offset + self.amplitude * np.sin(tmp * 2 * np.pi * self.frequency / 1000. + 2 * np.pi * self.phase / 360)
         self.amplitudes[-1] = 0.0
 
 
@@ -237,10 +238,10 @@ class NoisyCurrentSource(NeuronCurrentSource, electrodes.NoisyCurrentSource):
         self._generate()
 
     def _generate(self):
-        ## Not efficient at all... Is there a way to have those vectors computed on the fly ?
-        ## Otherwise should have a buffer mechanism
+        # Not efficient at all... Is there a way to have those vectors computed on the fly ?
+        # Otherwise should have a buffer mechanism
         temp_num_t = int(round((self.stop - self.start) / max(self.dt, simulator.state.dt)))
-        self.times = self.start + max(self.dt, simulator.state.dt) * numpy.arange(temp_num_t)
-        self.times = numpy.append(self.times, self.stop)
-        self.amplitudes = self.mean + self.stdev * numpy.random.randn(len(self.times))
+        self.times = self.start + max(self.dt, simulator.state.dt) * np.arange(temp_num_t)
+        self.times = np.append(self.times, self.stop)
+        self.amplitudes = self.mean + self.stdev * np.random.randn(len(self.times))
         self.amplitudes[-1] = 0.0
