@@ -1,17 +1,11 @@
 """
 Parameter set handling
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 """
 
-try:  # Python 2
-    basestring
-    long
-except NameError:  # Python 3
-    basestring = str
-    long = int
-import numpy
+import numpy as np
 import collections
 from pyNN.core import is_listlike
 from pyNN import errors
@@ -35,7 +29,7 @@ class LazyArray(larray):
 
     Arguments:
         `value`:
-            may be an int, long, float, bool, NumPy array, iterator, generator
+            may be an int, float, bool, NumPy array, iterator, generator
             or a function, `f(i)` or `f(i,j)`, depending on the dimensions of
             the array. `f(i,j)` should return a single number when `i` and `j`
             are integers, and a 1D array when either `i` or `j` or both is a
@@ -50,7 +44,7 @@ class LazyArray(larray):
     # the plan is ultimately to move everything to lazyarray
 
     def __init__(self, value, shape=None, dtype=None):
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             errmsg = "Value should be a string expressing a function of d. "
             try:
                 value = eval("lambda d: %s" % value)
@@ -65,7 +59,7 @@ class LazyArray(larray):
     def __setitem__(self, addr, new_value):
         self.check_bounds(addr)
         if (self.is_homogeneous
-            and isinstance(new_value, (int, long, float, bool))
+            and isinstance(new_value, (int, float, bool))
             and self.evaluate(simplify=True) == new_value):
             pass
         else:
@@ -80,16 +74,17 @@ class LazyArray(larray):
 
         `mask`: either `None` or a boolean array indicating which columns should be included.
         """
-        column_indices = numpy.arange(self.ncols)
+        column_indices = np.arange(self.ncols)
         if mask is not None:
-            assert len(mask) == self.ncols
+            if not isinstance(mask, slice):
+                assert len(mask) == self.ncols
             column_indices = column_indices[mask]
         if isinstance(self.base_value, RandomDistribution) and self.base_value.rng.parallel_safe:
             if mask is None:
                 for j in column_indices:
                     yield self._partially_evaluate((slice(None), j), simplify=True)
             else:
-                column_indices = numpy.arange(self.ncols)
+                column_indices = np.arange(self.ncols)
                 for j, local in zip(column_indices, mask):
                     col = self._partially_evaluate((slice(None), j), simplify=True)
                     if local:
@@ -117,11 +112,11 @@ class ArrayParameter(object):
     def __init__(self, value):
         if isinstance(value, ArrayParameter):
             self.value = value.value
-        elif isinstance(value, numpy.ndarray):
+        elif isinstance(value, np.ndarray):
             # dont overwrite dtype of int arrays
             self.value = value
         else:
-            self.value = numpy.array(value, float)
+            self.value = np.array(value, float)
 
     # def __len__(self):
     #     This must not be defined, otherwise ArrayParameter is insufficiently different from NumPy array
@@ -140,7 +135,8 @@ class ArrayParameter(object):
         element `i` of val.
         """
         if hasattr(val, '__len__'):
-            return numpy.array([self.__class__(self.value + x) for x in val], dtype=self.__class__)  # reshape if necessary?
+            # reshape if necessary?
+            return np.array([self.__class__(self.value + x) for x in val], dtype=self.__class__)
         else:
             return self.__class__(self.value + val)
 
@@ -154,7 +150,8 @@ class ArrayParameter(object):
         element `i` of val subtracted from it.
         """
         if hasattr(val, '__len__'):
-            return numpy.array([self.__class__(self.value - x) for x in val], dtype=self.__class__)  # reshape if necessary?
+            # reshape if necessary?
+            return np.array([self.__class__(self.value - x) for x in val], dtype=self.__class__)
         else:
             return self.__class__(self.value - val)
 
@@ -168,7 +165,8 @@ class ArrayParameter(object):
         element `i` of `val`.
         """
         if hasattr(val, '__len__'):
-            return numpy.array([self.__class__(self.value * x) for x in val], dtype=self.__class__)  # reshape if necessary?
+            # reshape if necessary?
+            return np.array([self.__class__(self.value * x) for x in val], dtype=self.__class__)
         else:
             return self.__class__(self.value * val)
 
@@ -184,17 +182,18 @@ class ArrayParameter(object):
         element `i` of `val`.
         """
         if hasattr(val, '__len__'):
-            return numpy.array([self.__class__(self.value / x) for x in val], dtype=self.__class__)  # reshape if necessary?
+            # reshape if necessary?
+            return np.array([self.__class__(self.value / x) for x in val], dtype=self.__class__)
         else:
             return self.__class__(self.value / val)
 
-    __truediv__ = __div__  # Python 3
+    __truediv__ = __div__
 
     def __eq__(self, other):
         if isinstance(other, ArrayParameter):
             return self.value.size == other.value.size and (self.value == other.value).all()
-        elif isinstance(other, numpy.ndarray) and other.size > 0 and isinstance(other[0], ArrayParameter):
-            return numpy.array([(self == seq).all() for seq in other])
+        elif isinstance(other, np.ndarray) and other.size > 0 and isinstance(other[0], ArrayParameter):
+            return np.array([(self == seq).all() for seq in other])
         else:
             return False
 
@@ -240,6 +239,7 @@ class ParameterSpace(object):
 
     .. _`lazy array`: https://lazyarray.readthedocs.org/
     """
+
     def __init__(self, parameters, schema=None, shape=None, component=None):
         """
 
@@ -270,10 +270,7 @@ class ParameterSpace(object):
 
         Note that the values will all be :class:`LazyArray` objects.
         """
-        if hasattr(self._parameters, "iteritems"):
-            return self._parameters.iteritems()
-        else:
-            return self._parameters.items()
+        return self._parameters.items()
 
     def __repr__(self):
         return "<ParameterSpace %s, shape=%s>" % (", ".join(self.keys()), self.shape)
@@ -311,9 +308,11 @@ class ParameterSpace(object):
                     self._parameters[name] = LazyArray(value, shape=self._shape,
                                                        dtype=expected_dtype)
                 except (TypeError, errors.InvalidParameterValueError):
-                    raise errors.InvalidParameterValueError("For parameter %s expected %s, got %s" % (name, expected_dtype, type(value)))
+                    raise errors.InvalidParameterValueError(
+                        "For parameter %s expected %s, got %s" % (name, expected_dtype, type(value)))
                 except ValueError as err:
-                    raise errors.InvalidDimensionsError(err)  # maybe put the more specific error classes into lazyarray
+                    # maybe put the more specific error classes into lazyarray
+                    raise errors.InvalidDimensionsError(err)
         else:
             for name, value in parameters.items():
                 self._parameters[name] = LazyArray(value, shape=self._shape)
@@ -356,9 +355,12 @@ class ParameterSpace(object):
             self._evaluated_shape = self._shape
         else:
             for name, value in self._parameters.items():
-                if isinstance(value.base_value, RandomDistribution) and value.base_value.rng.parallel_safe:
-                    value = value.evaluate()  # can't partially evaluate if using parallel safe
-                self._parameters[name] = value[mask]
+                try:
+                    if isinstance(value.base_value, RandomDistribution) and value.base_value.rng.parallel_safe:
+                        value = value.evaluate()  # can't partially evaluate if using parallel safe
+                    self._parameters[name] = value[mask]
+                except ValueError:
+                    raise errors.InvalidParameterValueError(f"{name} should not be of type {type(value)}")
             self._evaluated_shape = partial_shape(mask, self._shape)
         self._evaluated = True
         # should possibly update self.shape according to mask?
@@ -426,7 +428,8 @@ class ParameterSpace(object):
                         D[name] = value[:, j]
                     else:
                         D[name] = value
-                    assert not isinstance(D[name], LazyArray)  # should all have been evaluated by now
+                    # should all have been evaluated by now
+                    assert not isinstance(D[name], LazyArray)
                 yield D
 
     def __eq__(self, other):
@@ -462,8 +465,8 @@ class ParameterSpace(object):
         New array values are set to NaN.
         """
         for name, value in self._parameters.items():
-            if isinstance(value.base_value, numpy.ndarray):
-                new_base_value = numpy.ones(new_shape) * numpy.nan
+            if isinstance(value.base_value, np.ndarray):
+                new_base_value = np.ones(new_shape) * np.nan
                 new_base_value[mask] = value.base_value
                 self._parameters[name].base_value = new_base_value
         self.shape = new_shape
@@ -474,7 +477,7 @@ def simplify(value):
     If `value` is a homogeneous array, return the single value that all elements
     share. Otherwise, pass the value through.
     """
-    if isinstance(value, numpy.ndarray):
+    if isinstance(value, np.ndarray) and len(value.shape) > 0:  #  latter condition is for Brian scalar quantities
         if (value == value[0]).all():
             return value[0]
         else:
@@ -482,7 +485,7 @@ def simplify(value):
     else:
         return value
     # alternative - need to benchmark
-    #if numpy.any(arr != arr[0]):
+    # if np.any(arr != arr[0]):
     #    return arr
-    #else:
+    # else:
     #    return arr[0]
