@@ -6,11 +6,12 @@ formatting. If you need to produce more complex and/or publication-quality
 figures, it will probably be easier to use matplotlib or another plotting
 package directly rather than trying to extend this module.
 
-:copyright: Copyright 2006-2020 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2021 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
 """
 
+import sys
 from collections import defaultdict
 from numbers import Number
 from itertools import repeat
@@ -20,10 +21,6 @@ import matplotlib.gridspec as gridspec
 import numpy as np
 from quantities import ms
 from neo import AnalogSignal, SpikeTrain
-try:
-    from sys import maxint
-except ImportError:  # Py3
-    from sys import maxsize as maxint
 
 
 DEFAULT_FIG_SETTINGS = {
@@ -61,11 +58,11 @@ def plot_signal(ax, signal, index=None, label='', **options):
                                              signal.units._dimensionality.string)
     handle_options(ax, options)
     if index is None:
-        label = "%s (Neuron %d)" % (label, signal.channel_index or 0)
+        label = "%s (Neuron %d)" % (label, signal.array_annotations["channel_index"] or 0)
     else:
-        label = "%s (Neuron %d)" % (label, signal.channel_index[index])
+        label = "%s (Neuron %d)" % (label, signal.array_annotations["channel_index"][index])
         signal = signal[:, index]
-    ax.plot(signal.times.rescale(ms), signal, label=label, **options)
+    ax.plot(signal.times.rescale(ms), signal.magnitude, label=label, **options)
     ax.legend()
 
 
@@ -80,8 +77,8 @@ def plot_signals(ax, signal_array, label_prefix='', **options):
     handle_options(ax, options)
     offset = options.pop("y_offset", None)
     show_legend = options.pop("legend", True)
-    for i in signal_array.channel_index.index.argsort():
-        channel = signal_array.channel_index.index[i]
+    for i in signal_array.array_annotations["channel_index"].argsort():
+        channel = signal_array.array_annotations["channel_index"][i]
         signal = signal_array[:, i]
         if label_prefix:
             label = "%s (Neuron %d)" % (label_prefix, channel)
@@ -89,7 +86,7 @@ def plot_signals(ax, signal_array, label_prefix='', **options):
             label = "Neuron %d" % channel
         if offset:
             signal += i * offset
-        ax.plot(signal.times.rescale(ms), signal, label=label, **options)
+        ax.plot(signal.times.rescale(ms), signal.magnitude, label=label, **options)
     if show_legend:
         ax.legend()
 
@@ -101,11 +98,11 @@ def plot_spiketrains(ax, spiketrains, label='', **options):
     ax.set_xlim(0, spiketrains[0].t_stop / ms)
     handle_options(ax, options)
     max_index = 0
-    min_index = maxint
+    min_index = sys.maxsize
     for spiketrain in spiketrains:
         ax.plot(spiketrain,
-                 np.ones_like(spiketrain) * spiketrain.annotations['source_index'],
-                 'k.', **options)
+                np.ones_like(spiketrain) * spiketrain.annotations['source_index'],
+                'k.', **options)
         max_index = max(max_index, spiketrain.annotations['source_index'])
         min_index = min(min_index, spiketrain.annotations['source_index'])
     ax.set_ylabel("Neuron index")
@@ -195,7 +192,7 @@ class Figure(object):
         if "annotations" in options:
             gs.update(bottom=1.2 / height)  # leave space for annotations
         gs.update(top=1 - 0.8 / height, hspace=0.25)
-        #print(gs.get_grid_positions(self.fig))
+        # print(gs.get_grid_positions(self.fig))
 
         for i, panel in enumerate(panels):
             panel.plot(plt.subplot(gs[i, 0]))
@@ -215,6 +212,9 @@ class Figure(object):
             makedirs(dirname)
         self.fig.savefig(filename)
 
+    def show(self):
+        plt.show()
+
 
 class Panel(object):
     """
@@ -291,8 +291,8 @@ def comparison_plot(segments, labels, title='', annotations=None,
                 units[array.name] = array.units
             elif array.units != units[array.name]:
                 array = array.rescale(units[array.name])
-            for i in array.channel_index.index.argsort():
-                channel = array.channel_index.index[i]
+            for i in array.array_annotations["channel_index"].argsort():
+                channel = array.array_annotations["channel_index"][i]
                 signal = array[:, i]
                 by_var_and_channel[array.name][channel].append(signal)
     # each panel plots the signals for a given variable.
