@@ -2,12 +2,9 @@
 Unit tests for pyNN/random.py.
 """
 
-import pyNN.random as random
-import numpy
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+import unittest
+import numpy as np
+from numpy.testing import assert_allclose
 
 try:
     from neuron import h
@@ -17,13 +14,7 @@ else:
     have_nrn = True
     from pyNN.neuron.random import NativeRNG
 
-
-def assert_arrays_almost_equal(a, b, threshold):
-    if not (abs(a - b) < threshold).all():
-        err_msg = "%s != %s" % (a, b)
-        err_msg += "\nlargest difference = %g" % abs(a - b).max()
-        raise unittest.TestCase.failureException(err_msg)
-
+import pyNN.random as random
 
 # ==============================================================================
 class SimpleTests(unittest.TestCase):
@@ -48,9 +39,9 @@ class SimpleTests(unittest.TestCase):
     def testNextOne(self):
         """Calling next() with n=1 should return an array."""
         for rng in self.rnglist:
-            self.assertIsInstance(rng.next(1, 'uniform', {'low': 0, 'high': 1}), numpy.ndarray)
+            self.assertIsInstance(rng.next(1, 'uniform', {'low': 0, 'high': 1}), np.ndarray)
             self.assertIsInstance(rng.next(n=1, distribution='uniform',
-                                           parameters={'low': 0, 'high': 1}), numpy.ndarray)
+                                           parameters={'low': 0, 'high': 1}), np.ndarray)
             self.assertEqual(rng.next(1, distribution='uniform',
                                       parameters={'low': 0, 'high': 1}).shape, (1,))
 
@@ -110,8 +101,8 @@ class ParallelTests(unittest.TestCase):
             self.assertEqual(rng0.seed, 1000)
             self.assertEqual(rng1.seed, 1001)
             self.assertEqual(rng_check.seed, 1001)
-            mask1 = numpy.array((1, 0, 1, 0, 1), bool)
-            mask2 = numpy.array((0, 1, 0, 1, 0), bool)
+            mask1 = np.array((1, 0, 1, 0, 1), bool)
+            mask2 = np.array((0, 1, 0, 1, 0), bool)
             draw0 = rng0.next(5, 'uniform', {'low': 0, 'high': 1}, mask=mask1)
             draw1 = rng1.next(5, 'uniform', {'low': 0, 'high': 1}, mask=mask2)
             draw_check = rng_check.next(5, 'uniform', {'low': 0, 'high': 1}, mask=None)
@@ -130,8 +121,8 @@ class ParallelTests(unittest.TestCase):
             self.assertEqual(rng0.seed, 1000)
             self.assertEqual(rng1.seed, 1000)
             self.assertEqual(rng_check.seed, 1000)
-            mask1 = numpy.array((1, 0, 1, 0, 1), bool)
-            mask2 = numpy.array((0, 1, 0, 1, 0), bool)
+            mask1 = np.array((1, 0, 1, 0, 1), bool)
+            mask2 = np.array((0, 1, 0, 1, 0), bool)
             draw0 = rng0.next(5, 'uniform', {'low': 0, 'high': 1}, mask=mask1)
             draw1 = rng1.next(5, 'uniform', {'low': 0, 'high': 1}, mask=mask2)
             draw_check = rng_check.next(5, 'uniform', {'low': 0, 'high': 1}, mask=None)
@@ -161,7 +152,7 @@ class ParallelTests(unittest.TestCase):
         A = range(10)
         perm0 = rng0.permutation(A)
         perm1 = rng1.permutation(A)
-        assert_arrays_almost_equal(perm0, perm1, 1e-99)
+        assert_allclose(perm0, perm1, 1e-99)
 
 
 class NativeRNGTests(unittest.TestCase):
@@ -237,7 +228,7 @@ class RandomDistributionTests(unittest.TestCase):
         # number of redraws. This should be caught.
         for rng in self.rnglist:
             rd1 = random.RandomDistribution(
-                'normal_clipped', mu=0, sigma=1, low=5, high=numpy.inf, rng=rng)
+                'normal_clipped', mu=0, sigma=1, low=5, high=np.inf, rng=rng)
             self.assertRaises(Exception, rd1.next, 1000)
 
 
